"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandHandler = void 0;
const discord_js_1 = require("discord.js");
const MessageBuilder_1 = require("../structures/MessageBuilder");
class CommandHandler {
    constructor(client) {
        this.commands = new Map;
        this.client = client;
        this.client.on('interactionCreate', (interaction) => this.handleCommand(interaction));
    }
    loadCommands() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!((_b = (_a = this.client) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.commandsPath)) {
                return;
            }
            const commands = yield this.client.getContextValues(this.client.options.commandsPath);
            this.commands = new Map(commands.map(command => [command.name, command]));
        });
    }
    handleCommand(interaction) {
        return __awaiter(this, void 0, void 0, function* () {
            if (interaction.type !== discord_js_1.InteractionType.ApplicationCommand) {
                return;
            }
            if (!this.commands) {
                return;
            }
            const command = this.commands.get(interaction.commandName);
            if (!command) {
                return;
            }
            yield interaction.deferReply();
            const response = yield command.execute({
                command: interaction.commandName,
                input: interaction.options.data.reduce((acc, option) => (Object.assign(Object.assign({}, acc), { [option.name]: option.value })), {}),
                guild: interaction.guild,
                user: interaction.user,
                channel: interaction === null || interaction === void 0 ? void 0 : interaction.channel,
                member: interaction === null || interaction === void 0 ? void 0 : interaction.member,
                interaction: interaction,
            });
            if (!response) {
                yield interaction.deleteReply();
                return;
            }
            const isObject = typeof response === "object";
            if (response instanceof MessageBuilder_1.MessageBuilder) {
                yield (response === null || response === void 0 ? void 0 : response.followUp(interaction));
            }
            else {
                yield interaction.editReply(isObject ? { embeds: [response] } : response);
            }
        });
    }
}
exports.CommandHandler = CommandHandler;
