"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TwokeiClient = void 0;
const discord_js_1 = require("discord.js");
const CommandHandler_1 = require("../handlers/CommandHandler");
const EventHandler_1 = require("../handlers/EventHandler");
const fast_glob_1 = __importDefault(require("fast-glob"));
class TwokeiClient extends discord_js_1.Client {
    /**
     * @param twokeiOptions
     */
    constructor(twokeiOptions) {
        super(Object.assign({ autoload: true }, twokeiOptions));
        this.eventHandler = new EventHandler_1.EventHandler(this);
        this.commandHandler = new CommandHandler_1.CommandHandler(this);
        if (twokeiOptions.autoload) {
            this.commandHandler.loadCommands().then(() => this.registerSlashCommands());
            this.eventHandler.loadEvents();
        }
    }
    getContextValues(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield this.getAllFiles(path);
            const values = files.map(file => {
                var _a;
                const command = require(`${(_a = this.options.currentWorkingDirectory) !== null && _a !== void 0 ? _a : process.cwd()}/${file}`);
                return Object.values(command);
            });
            return values.flat();
        });
    }
    getAllFiles(path) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const cwd = (_a = this.options.currentWorkingDirectory) !== null && _a !== void 0 ? _a : process.cwd();
            return yield (0, fast_glob_1.default)(path, { cwd, onlyFiles: true });
        });
    }
    registerSlashCommands() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const clientId = (_b = (_a = this.application) === null || _a === void 0 ? void 0 : _a.id) !== null && _b !== void 0 ? _b : process.env.CLIENT_ID;
            if (!clientId || !process.env.TOKEN) {
                throw new Error('Missing CLIENT_ID, TOKEN or GUILD_ID in .env file.');
            }
            const parsed = Array.from(this.commandHandler.commands.values()).map(command => this.parseCommandToSlashJSON(command));
            const rest = new discord_js_1.REST({ version: '10' }).setToken(process.env.TOKEN);
            if (process.env.NODE_ENV !== 'production' && process.env.GUILD_ID) {
                // rest.put(Routes.applicationGuildCommands(clientId, process.env.GUILD_ID), { body: [] })
                // 	.then(() => console.log('Successfully deleted all guild commands.'))
                // 	.catch(console.error);
                yield rest.put(discord_js_1.Routes.applicationGuildCommands(clientId, process.env.GUILD_ID), { body: parsed }).then(() => console.log('Successfully registered guild commands.'))
                    .catch(console.error);
            }
            else {
                yield rest.put(discord_js_1.Routes.applicationCommands(clientId), { body: parsed }).then(() => console.log('Successfully registered application commands.'))
                    .catch(console.error);
            }
        });
    }
    parseCommandToSlashJSON(command) {
        var _a, _b, _c, _d, _e, _f, _g;
        const name = (_b = (_a = command.nameLocales) === null || _a === void 0 ? void 0 : _a['en-US']) !== null && _b !== void 0 ? _b : command.name;
        const description = (_e = (_d = (_c = command.descriptionLocales) === null || _c === void 0 ? void 0 : _c['en-US']) !== null && _d !== void 0 ? _d : command.description) !== null && _e !== void 0 ? _e : 'No description provided.';
        if (!name || !description) {
            throw new Error(`Command ${command.name} is missing a name or description for en-US locale.`);
        }
        const previewSlash = new discord_js_1.SlashCommandBuilder()
            .setName(name)
            .setDescription(description)
            .setDefaultMemberPermissions(command.permissions)
            .setNameLocalizations(command.nameLocales || {})
            .setDescriptionLocalizations(command.descriptionLocales || {});
        const slash = (_g = (_f = command.slash) === null || _f === void 0 ? void 0 : _f.call(command, previewSlash)) !== null && _g !== void 0 ? _g : previewSlash;
        return slash.toJSON();
    }
}
exports.TwokeiClient = TwokeiClient;
